USE [master]
GO
IF (DB_ID(N'StairwayToSQLCLR') IS NULL)
BEGIN
	PRINT 'Creating test DB...';
	CREATE DATABASE [StairwayToSQLCLR];
END

IF (EXISTS(
			SELECT	*
			FROM	sys.databases sd
			WHERE	sd.[name] = N'StairwayToSQLCLR'
			AND		(
						sd.recovery_model <> 3
					OR	sd.is_broker_enabled = 1
					)
			)
	)
BEGIN
	PRINT 'Altering test DB...'
	ALTER DATABASE [StairwayToSQLCLR] SET RECOVERY SIMPLE, DISABLE_BROKER WITH ROLLBACK IMMEDIATE;
END
GO
------------------------------
USE [StairwayToSQLCLR]
GO

SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
SET NOCOUNT ON
GO

IF (OBJECT_ID('dbo.StairwayToSQLCLR_02') IS NULL)
BEGIN
	PRINT 'Creating test table...'
	CREATE TABLE dbo.StairwayToSQLCLR_02 (
											ID UNIQUEIDENTIFIER NOT NULL PRIMARY KEY CLUSTERED,
											InsertTime DATETIME NOT NULL
										)
END

IF (NOT EXISTS(SELECT * FROM dbo.StairwayToSQLCLR_02))
BEGIN
	PRINT 'Inserting test data...'
	INSERT INTO dbo.StairwayToSQLCLR_02 (ID, InsertTime)
		SELECT	TOP 100 NEWID() AS [ID], GETDATE() AS [InsertTime]
		FROM	sys.all_columns
END


-- DROP PROCEDURE dbo.StairwayToSQLCLR_02_GetNumRows
IF (OBJECT_ID('dbo.StairwayToSQLCLR_02_GetNumRows') IS NULL)
BEGIN
	PRINT 'Creating support proc...'
	EXEC('
/*
@CountMethod options:
3 = check @TableName via system table
4 = check @TableName directly via Dynamic SQL
5 = check StairwayToSQLCLR_02 directly in regular SQL without SET NOCOUNT or NEWID()
6 = check StairwayToSQLCLR_02 directly in regular SQL but attempt SET NOCOUNT ON
7 = check StairwayToSQLCLR_02 directly in regular SQL but attempt using NEWID()
8 = check #Temp temp table
9 = return CONTEXT_INFO() converted to BIGINT
*/
CREATE PROCEDURE dbo.StairwayToSQLCLR_02_GetNumRows
(
	@TableName SYSNAME,
	@CountMethod TINYINT = 3 -- check via system table
)
AS

IF (@CountMethod NOT BETWEEN 3 AND 9)
BEGIN
	RAISERROR(N''@CountMethod needs to be between 3 and 9.'', 16, 1)
	RETURN
END


IF (@CountMethod = 3)
BEGIN
	SELECT SUM([rows]) AS [TotalRows]
	FROM sys.partitions sp
	WHERE sp.[object_id] = OBJECT_ID(@TableName)
	AND sp.index_id < 2

	RETURN
END

IF (@CountMethod = 4)
BEGIN
	DECLARE @SQL NVARCHAR(4000)
	SET @SQL = N''SELECT COUNT_BIG(*) AS [TotalRows]
		FROM '' + @TableName + N'' WITH (NOLOCK)''
	EXEC(@SQL)
	--EXEC sp_executesql @SQL

	RETURN
END

IF (@CountMethod = 6)
BEGIN
	SET NOCOUNT ON
	--SET DATEFIRST 3
END

IF (@CountMethod = 7)
BEGIN
	DECLARE @NewGUID UNIQUEIDENTIFIER
	SET @NewGUID = NEWID()
END

IF (@CountMethod = 8)
BEGIN
	SELECT COUNT_BIG(*) AS [TotalRows]
	FROM #Temp

	RETURN
END

IF (@CountMethod = 9)
BEGIN
	SELECT COALESCE(CONVERT(BIGINT, CONVERT(VARBINARY(5), CONTEXT_INFO())), -1) AS [TotalRows]

	RETURN
END


-- @CountMethod = 5 skips the IF blocks and only runs the following:
SELECT COUNT_BIG(*) AS [TotalRows]
FROM dbo.StairwayToSQLCLR_02 WITH (NOLOCK)

')
END


GO
